@@ -14,7 +14,7 @@ use staticvec::*;
 
 extern crate test;
 
-use test::Bencher;
+use test::{black_box, Bencher};
 
 const VEC_SIZE: usize = 16;
 const SPILLED_SIZE: usize = 100;
@@ -125,50 +125,50 @@ macro_rules! make_benches {
 
 make_benches! {
   StaticVec<u64, {SPILLED_SIZE}> {
-    bench_push => gen_push(SPILLED_SIZE as _),
-    bench_push_small => gen_push(VEC_SIZE as _),
-    bench_insert => gen_insert(SPILLED_SIZE_TWO as _),
-    bench_insert_small => gen_insert(VEC_SIZE as _),
-    bench_remove => gen_remove(SPILLED_SIZE as _),
-    bench_remove_small => gen_remove(VEC_SIZE as _),
-    bench_extend => gen_extend(SPILLED_SIZE as _),
-    bench_extend_small => gen_extend(VEC_SIZE as _),
-    bench_from_iter => gen_from_iter(SPILLED_SIZE as _),
-    bench_from_iter_small => gen_from_iter(VEC_SIZE as _),
-    bench_from_slice => gen_from_slice(SPILLED_SIZE as _),
-    bench_from_slice_small => gen_from_slice(VEC_SIZE as _),
-    bench_extend_from_slice => gen_extend_from_slice(SPILLED_SIZE as _),
-    bench_extend_from_slice_small => gen_extend_from_slice(VEC_SIZE as _),
-    bench_macro_from_elem => gen_from_elem(SPILLED_SIZE as _),
-    bench_pushpop => gen_pushpop(),
+    staticvec_bench_push => gen_push(SPILLED_SIZE as _),
+    staticvec_bench_push_small => gen_push(VEC_SIZE as _),
+    staticvec_bench_insert => gen_insert(SPILLED_SIZE_TWO as _),
+    staticvec_bench_insert_small => gen_insert(VEC_SIZE as _),
+    staticvec_bench_remove => gen_remove(SPILLED_SIZE as _),
+    staticvec_bench_remove_small => gen_remove(VEC_SIZE as _),
+    staticvec_bench_extend => gen_extend(SPILLED_SIZE as _),
+    staticvec_bench_extend_small => gen_extend(VEC_SIZE as _),
+    staticvec_bench_from_iter => gen_from_iter(SPILLED_SIZE as _),
+    staticvec_bench_from_iter_small => gen_from_iter(VEC_SIZE as _),
+    staticvec_bench_from_slice => gen_from_slice(SPILLED_SIZE as _),
+    staticvec_bench_from_slice_small => gen_from_slice(VEC_SIZE as _),
+    staticvec_bench_extend_from_slice => gen_extend_from_slice(SPILLED_SIZE as _),
+    staticvec_bench_extend_from_slice_small => gen_extend_from_slice(VEC_SIZE as _),
+    staticvec_bench_macro_from_elem => gen_from_elem(SPILLED_SIZE as _),
+    staticvec_bench_pushpop => gen_pushpop(),
   }
 }
 
 make_benches! {
   StaticVec<u64, {VEC_SIZE}> {
-    bench_macro_from_elem_small => gen_from_elem(VEC_SIZE as _),
+    staticvec_bench_macro_from_elem_small => gen_from_elem(VEC_SIZE as _),
   }
 }
 
 make_benches! {
   Vec<u64> {
-    bench_push_vec => gen_push(SPILLED_SIZE as _),
-    bench_push_vec_small => gen_push(VEC_SIZE as _),
-    bench_insert_vec => gen_insert(SPILLED_SIZE_TWO as _),
-    bench_insert_vec_small => gen_insert(VEC_SIZE as _),
-    bench_remove_vec => gen_remove(SPILLED_SIZE as _),
-    bench_remove_vec_small => gen_remove(VEC_SIZE as _),
-    bench_extend_vec => gen_extend(SPILLED_SIZE as _),
-    bench_extend_vec_small => gen_extend(VEC_SIZE as _),
-    bench_from_iter_vec => gen_from_iter(SPILLED_SIZE as _),
-    bench_from_iter_vec_small => gen_from_iter(VEC_SIZE as _),
-    bench_from_slice_vec => gen_from_slice(SPILLED_SIZE as _),
-    bench_from_slice_vec_small => gen_from_slice(VEC_SIZE as _),
-    bench_extend_from_slice_vec => gen_extend_from_slice(SPILLED_SIZE as _),
-    bench_extend_from_slice_vec_small => gen_extend_from_slice(VEC_SIZE as _),
-    bench_macro_from_elem_vec => gen_from_elem(SPILLED_SIZE as _),
-    bench_macro_from_elem_vec_small => gen_from_elem(VEC_SIZE as _),
-    bench_pushpop_vec => gen_pushpop(),
+    vec_bench_push => gen_push(SPILLED_SIZE as _),
+    vec_bench_push_small => gen_push(VEC_SIZE as _),
+    vec_bench_insert => gen_insert(SPILLED_SIZE_TWO as _),
+    vec_bench_insert_small => gen_insert(VEC_SIZE as _),
+    vec_bench_remove => gen_remove(SPILLED_SIZE as _),
+    vec_bench_remove_small => gen_remove(VEC_SIZE as _),
+    vec_bench_extend => gen_extend(SPILLED_SIZE as _),
+    vec_bench_extend_small => gen_extend(VEC_SIZE as _),
+    vec_bench_from_iter => gen_from_iter(SPILLED_SIZE as _),
+    vec_bench_from_iter_small => gen_from_iter(VEC_SIZE as _),
+    vec_bench_from_slice => gen_from_slice(SPILLED_SIZE as _),
+    vec_bench_from_slice_small => gen_from_slice(VEC_SIZE as _),
+    vec_bench_extend_from_slice => gen_extend_from_slice(SPILLED_SIZE as _),
+    vec_bench_extend_from_slice_small => gen_extend_from_slice(VEC_SIZE as _),
+    vec_bench_macro_from_elem => gen_from_elem(SPILLED_SIZE as _),
+    vec_bench_macro_from_elem_small => gen_from_elem(VEC_SIZE as _),
+    vec_bench_pushpop => gen_pushpop(),
   }
 }
 
@@ -278,7 +278,7 @@ fn gen_from_elem<V: Vector<u64>>(n: usize, b: &mut Bencher) {
 }
 
 #[bench]
-fn bench_macro_from_list(b: &mut Bencher) {
+fn staticvec_bench_macro_from_list(b: &mut Bencher) {
   b.iter(|| {
     let vec = staticvec![
       0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 20, 24, 32, 36, 0x40, 0x80, 0x100,
@@ -290,7 +290,7 @@ fn bench_macro_from_list(b: &mut Bencher) {
 }
 
 #[bench]
-fn bench_macro_from_list_vec(b: &mut Bencher) {
+fn vec_bench_macro_from_list(b: &mut Bencher) {
   b.iter(|| {
     let vec: Vec<u64> = vec![
       0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 20, 24, 32, 36, 0x40, 0x80, 0x100,
@@ -300,3 +300,79 @@ fn bench_macro_from_list_vec(b: &mut Bencher) {
     vec
   });
 }
+
+#[bench]
+fn bench_clone(b: &mut Bencher) {
+  let v: StaticVec<Vec<u32>, { 200 }> = (0..100).map(|i| (0..i).collect()).collect();
+
+  b.iter(move || {
+    let _v2 = black_box(v.clone());
+  });
+}
+
+#[bench]
+fn bench_trivial_clone(b: &mut Bencher) {
+  let v: StaticVec<u32, { 200 }> = (50..150).collect();
+
+  b.iter(move || {
+    let _v2 = black_box(v.clone());
+  })
+}
+
+#[bench]
+fn bench_clone_from_shorter(b: &mut Bencher) {
+  // We create some vectors with semi-random lengths to provoke
+  // different behaviors in the underlying Vec::clone_from. This allows us to
+  // demonstrate the advantage of using an underlying clone_from over a raw
+  // clone.
+  let src: StaticVec<Vec<u32>, { 200 }> = (0..50).map(|i| (0..i % 7).collect()).collect();
+
+  b.iter(move || {
+    // TODO: find a way to make the bencher not include the time required to
+    // instantiate `dst`. We can't move it outside of b.iter because we need
+    // to make sure dst is in the same state before each bench run. We don't
+    // want to clone it, either, because part of the test includes the
+    // allocations into `vec`
+    let mut dst: StaticVec<Vec<u32>, { 200 }> = black_box(
+      (0..100)
+        .map(|i| {
+          // ensure we have enough capacity to benefit from clone_from
+          let mut vec = Vec::with_capacity(20);
+          vec.extend(1..1 + (i % 11));
+          vec
+        })
+        .collect(),
+    );
+
+    dst.clone_from(&src);
+  });
+}
+
+#[bench]
+fn bench_clone_from_longer(b: &mut Bencher) {
+  // We create some vectors with semi-random lengths to provoke
+  // different behaviors in the underlying Vec::clone_from. This allows us to
+  // demonstrate the advantage of using an underlying clone_from over a raw
+  // clone.
+  let src: StaticVec<Vec<u32>, { 200 }> = (0..100).map(|i| (0..i % 7).collect()).collect();
+
+  b.iter(move || {
+    // TODO: find a way to make the bencher not include the time required to
+    // instantiate `dst`. We can't move it outside of b.iter because we need
+    // to make sure dst is in the same state before each bench run. We don't
+    // want to clone it, either, because part of the test includes the
+    // allocations into `vec`
+    let mut dst: StaticVec<Vec<u32>, { 200 }> = black_box(
+      (0..50)
+        .map(|i| {
+          // ensure we have enouch capacity to benefit from clone_from
+          let mut vec = Vec::with_capacity(20);
+          vec.extend(1..1 + (i % 11));
+          vec
+        })
+        .collect(),
+    );
+
+    dst.clone_from(&src);
+  });
+}
