@@ -9,6 +9,7 @@
 #![feature(exact_size_is_empty)]
 #![feature(trusted_len)]
 #![feature(slice_partition_dedup)]
+#![feature(read_initializer)]
 
 //Literally just for stable-sort.
 #[cfg(any(feature = "std", rustdoc))]
@@ -29,6 +30,7 @@ use core::marker::PhantomData;
 use core::mem::{self, MaybeUninit};
 use core::ops::{Bound::Excluded, Bound::Included, Bound::Unbounded, RangeBounds};
 use core::ptr;
+use core::slice;
 
 mod iterators;
 #[macro_use]
@@ -156,7 +158,7 @@ impl<T, const N: usize> StaticVec<T, { N }> {
 
   ///Serves the same purpose as [capacity](crate::StaticVec::capacity), but as an associated
   ///constant rather than a method.
-  pub const CAPACITY: usize = Self::cap();
+  pub const CAPACITY: usize = N;
 
   /// Returns the remaining capacity of the `StaticVec`.
   #[inline(always)]
@@ -169,6 +171,12 @@ impl<T, const N: usize> StaticVec<T, { N }> {
   ///the potential for reading from unitialized memory later on.
   #[inline(always)]
   pub unsafe fn set_len(&mut self, new_len: usize) {
+    debug_assert!(
+      new_len <= N,
+      "Attempted to unsafely set length to {}; maximum is {}",
+      new_len,
+      N
+    );
     self.length = new_len;
   }
 
@@ -211,21 +219,58 @@ impl<T, const N: usize> StaticVec<T, { N }> {
   ///Returns a constant reference to a slice of the StaticVec's inhabited area.
   #[inline(always)]
   pub fn as_slice(&self) -> &[T] {
-    unsafe { &*(self.data.get_unchecked(0..self.length) as *const [MaybeUninit<T>] as *const [T]) }
+    // Safety: `self.as_ptr()` is a pointer to an array for which the first `length`
+    // elements are guaranteed to be initialized. Therefore this is a valid slice.
+    unsafe { slice::from_raw_parts(self.as_ptr(), self.length) }
   }
 
   ///Returns a mutable reference to a slice of the StaticVec's inhabited area.
   #[inline(always)]
   pub fn as_mut_slice(&mut self) -> &mut [T] {
-    unsafe {
-      &mut *(self.data.get_unchecked_mut(0..self.length) as *mut [MaybeUninit<T>] as *mut [T])
-    }
+    // Safety: See as_slice.
+    unsafe { slice::from_raw_parts_mut(self.as_mut_ptr(), self.length) }
+  }
+
+  ///Returns a constant reference to the element of the StaticVec at `index`, if `index` is within the range `0..length`.
+  ///No checks are performed to ensure that is the case, so this function is marked `unsafe` and should be used with caution
+  ///only when performance is absolutely paramount.
+  ///# Safety
+  ///It is up to the caller to ensure that `index` is within the appropriate bounds.
+  #[inline(always)]
+  pub unsafe fn get_unchecked(&self, index: usize) -> &T {
+    debug_assert!(
+      index < self.length,
+      "Attempted to unsafely get at index {} when length is {}",
+      index,
+      self.length
+    );
+    self.data.get_unchecked(index).get_ref()
+  }
+
+  ///Returns a mutable reference to the element of the StaticVec at `index`, if `index` is within the range `0..length`.
+  ///No checks are performed to ensure that is the case, so this function is marked `unsafe` and should be used with caution
+  ///only when performance is absolutely paramount.
+  ///# Safety
+  ///It is up to the caller to ensure that `index` is within the appropriate bounds.
+  #[inline(always)]
+  pub unsafe fn get_unchecked_mut(&mut self, index: usize) -> &mut T {
+    debug_assert!(
+      index < self.length,
+      "Attempted to unsafely get at index {} when length is {}",
+      index,
+      self.length
+    );
+    self.data.get_unchecked_mut(index).get_mut()
   }
 
   ///Appends a value to the end of the StaticVec without asserting that
   ///its current length is less than `N`.
   #[inline(always)]
   pub unsafe fn push_unchecked(&mut self, value: T) {
+    debug_assert!(
+      self.is_not_full(),
+      "Attempted to unsafely push to a full StaticVec"
+    );
     //Written this way to avoid the `&mut` return value (that we don't use)
     //of `MaybeUninit::write()`, which seems to have a strange effect on the optimizer
     //for this function.
@@ -241,20 +286,14 @@ impl<T, const N: usize> StaticVec<T, { N }> {
   ///the StaticVec's current length is greater than 0.
   #[inline(always)]
   pub unsafe fn pop_unchecked(&mut self) -> T {
+    debug_assert!(
+      self.is_not_empty(),
+      "Attempted to unsafely pop from an empty StaticVec"
+    );
     self.length -= 1;
     self.data.get_unchecked(self.length).read()
   }
 
-  ///Asserts that the current length of the StaticVec is less than `N`,
-  ///and if so appends a value to the end of it.
-  #[inline(always)]
-  pub fn push(&mut self, value: T) {
-    assert!(self.length < N);
-    unsafe {
-      self.push_unchecked(value);
-    }
-  }
-
   ///Pushes `value` to the StaticVec if its current length is less than its capacity,
   ///or returns an error indicating there's no remaining capacity otherwise.
   #[inline(always)]
@@ -269,6 +308,13 @@ impl<T, const N: usize> StaticVec<T, { N }> {
     }
   }
 
+  /// Push a value to the end of this `StaticVec`. Panics if the collection is
+  /// full; that is, if `self.len() == self.capacity()`.
+  #[inline(always)]
+  pub fn push(&mut self, value: T) {
+    self.try_push(value).unwrap()
+  }
+
   ///Removes the value at the last position of the StaticVec and returns it in `Some` if
   ///the StaticVec has a current length greater than 0, and returns `None` otherwise.
   #[inline(always)]
@@ -287,7 +333,7 @@ impl<T, const N: usize> StaticVec<T, { N }> {
     if self.length == 0 {
       None
     } else {
-      Some(unsafe { self.data.get_unchecked(0).get_ref() })
+      Some(unsafe { self.get_unchecked(0) })
     }
   }
 
@@ -298,7 +344,7 @@ impl<T, const N: usize> StaticVec<T, { N }> {
     if self.length == 0 {
       None
     } else {
-      Some(unsafe { self.data.get_unchecked_mut(0).get_mut() })
+      Some(unsafe { self.get_unchecked_mut(0) })
     }
   }
 
@@ -309,7 +355,7 @@ impl<T, const N: usize> StaticVec<T, { N }> {
     if self.length == 0 {
       None
     } else {
-      Some(unsafe { self.data.get_unchecked(self.length - 1).get_ref() })
+      Some(unsafe { self.get_unchecked(self.length - 1) })
     }
   }
 
@@ -320,7 +366,7 @@ impl<T, const N: usize> StaticVec<T, { N }> {
     if self.length == 0 {
       None
     } else {
-      Some(unsafe { self.data.get_unchecked_mut(self.length - 1).get_mut() })
+      Some(unsafe { self.get_unchecked_mut(self.length - 1) })
     }
   }
 
@@ -453,30 +499,6 @@ impl<T, const N: usize> StaticVec<T, { N }> {
     }
   }
 
-  ///Performs a stable in-place sort of the StaticVec's inhabited area.
-  ///Locally requires that `T` implements [Ord](core::cmp::Ord) to make the sorting possible.
-  #[cfg(feature = "std")]
-  #[doc(cfg(feature = "std"))]
-  #[inline(always)]
-  pub fn sort(&mut self)
-  where T: Ord {
-    self.as_mut_slice().sort();
-  }
-
-  ///Performs an unstable in-place sort of the StaticVec's inhabited area.
-  ///Locally requires that `T` implements [Ord](core::cmp::Ord) to make the sorting possible.
-  #[inline(always)]
-  pub fn sort_unstable(&mut self)
-  where T: Ord {
-    self.as_mut_slice().sort_unstable();
-  }
-
-  ///Reverses the contents of the StaticVec's inhabited area in-place.
-  #[inline(always)]
-  pub fn reverse(&mut self) {
-    self.as_mut_slice().reverse();
-  }
-
   ///Returns a separate, stable-sorted StaticVec of the contents of the
   ///StaticVec's inhabited area without modifying the original data.
   ///Locally requires that `T` implements [Copy](core::marker::Copy) to avoid soundness issues,
@@ -540,7 +562,8 @@ impl<T, const N: usize> StaticVec<T, { N }> {
   #[inline(always)]
   pub fn extend_from_slice(&mut self, other: &[T])
   where T: Copy {
-    let added_length = (self.length + other.len()).min(N - self.length);
+    let added_length = other.len().min(self.remaining_capacity());
+    // Safety: added_length is <= our remaining capacity and other.len.
     unsafe {
       other
         .as_ptr()
@@ -571,18 +594,20 @@ impl<T, const N: usize> StaticVec<T, { N }> {
   ///Returns a [Vec](alloc::vec::Vec) containing the contents of the StaticVec instance.
   ///The returned [Vec](alloc::vec::Vec) will initially have the same value for
   ///[len](alloc::vec::Vec::len) and [capacity](alloc::vec::Vec::capacity) as the source StaticVec.
+  ///Note that while using this function does *not* consume the source StaticVec in the sense of rendering
+  ///it completely inaccessible / unusable, it *does* empty it (that is, it will have no contents and
+  ///a `length` of 0 afterwards.)
   #[cfg(feature = "std")]
   #[doc(cfg(feature = "std"))]
   #[inline(always)]
   pub fn into_vec(&mut self) -> Vec<T> {
-    let mut res = Vec::<T>::with_capacity(N);
+    let mut res = Vec::with_capacity(N);
     unsafe {
       self
         .as_ptr()
         .copy_to_nonoverlapping(res.as_mut_ptr(), self.length);
       res.set_len(self.length);
       self.length = 0;
-      mem::forget(self.as_mut_slice());
       res
     }
   }
