@@ -6,17 +6,18 @@ use core::fmt::{self, Debug, Formatter};
 use core::hash::{Hash, Hasher};
 use core::iter::FromIterator;
 use core::mem::MaybeUninit;
-use core::ops::{Index, IndexMut, Range, RangeFull, RangeInclusive};
+use core::ops::{Deref, DerefMut, Index, IndexMut, Range, RangeFull, RangeInclusive};
 use core::ptr;
+use core::str;
 
-#[cfg(feature = "deref_to_slice")]
-use core::ops::{Deref, DerefMut};
+#[cfg(feature = "std")]
+use alloc::string::String;
 
 #[cfg(feature = "std")]
 use alloc::vec::Vec;
 
 #[cfg(feature = "std")]
-use std::io::{self, Error, ErrorKind, IoSlice, IoSliceMut, Read, Write};
+use std::io::{self, IoSlice, IoSliceMut, Read, Write};
 
 #[cfg(feature = "serde_support")]
 use core::marker::PhantomData;
@@ -46,16 +47,46 @@ impl<T: Clone, const N: usize> Clone for StaticVec<T, { N }> {
   fn clone(&self) -> Self {
     let mut res = Self::new();
     for i in 0..self.length {
+      // Safety: `self` has the same capacity as `res`, and `res` is
+      // empty, so all of the following writes are safe.
       unsafe {
+        //`push_unchecked` (and the previously used iterator) seem to have the same optimizer problem
+        //in this "hot loop" context as `filled_with` did generally. So for loop plus `get_unchecked` it is
+        //for the time being.
         res
           .data
           .get_unchecked_mut(i)
-          .write(self.data.get_unchecked(i).get_ref().clone());
+          .write(self.get_unchecked(i).clone());
         res.length += 1;
       }
     }
     res
   }
+
+  #[inline]
+  fn clone_from(&mut self, rhs: &Self) {
+    self.truncate(rhs.length);
+    for i in 0..self.length {
+      // Safety: after the truncate, self.len <= rhs.len, which means that for
+      // every i in self, there is definitely an element at rhs[i].
+      unsafe {
+        self.get_unchecked_mut(i).clone_from(rhs.get_unchecked(i));
+      }
+    }
+    for i in self.length..rhs.length {
+      // Safety: i < rhs.length, so rhs.get_unchecked is safe. i starts at
+      // self.length, which is <= rhs.length, so there is always an available
+      // slot at self[i] to write into.
+      unsafe {
+        //Same thing with `push_unchecked` here as for `clone`.
+        self
+          .data
+          .get_unchecked_mut(i)
+          .write(rhs.get_unchecked(i).clone());
+        self.length += 1;
+      }
+    }
+  }
 }
 
 impl<T: Debug, const N: usize> Debug for StaticVec<T, { N }> {
@@ -73,7 +104,6 @@ impl<T, const N: usize> Default for StaticVec<T, { N }> {
   }
 }
 
-#[cfg(feature = "deref_to_slice")]
 impl<T, const N: usize> Deref for StaticVec<T, { N }> {
   type Target = [T];
   #[inline(always)]
@@ -82,7 +112,6 @@ impl<T, const N: usize> Deref for StaticVec<T, { N }> {
   }
 }
 
-#[cfg(feature = "deref_to_slice")]
 impl<T, const N: usize> DerefMut for StaticVec<T, { N }> {
   #[inline(always)]
   fn deref_mut(&mut self) -> &mut [T] {
@@ -218,7 +247,7 @@ impl<T, const N: usize> IndexMut<Range<usize>> for StaticVec<T, { N }> {
 impl<T, const N: usize> Index<RangeFull> for StaticVec<T, { N }> {
   type Output = [T];
   ///Returns a constant reference to a slice consisting of `0..self.length`
-  //elements of the StaticVec, using [as_slice](crate::StaticVec::as_slice) internally.
+  ///elements of the StaticVec, using [as_slice](crate::StaticVec::as_slice) internally.
   #[inline(always)]
   fn index(&self, _index: RangeFull) -> &Self::Output {
     self.as_slice()
@@ -227,7 +256,7 @@ impl<T, const N: usize> Index<RangeFull> for StaticVec<T, { N }> {
 
 impl<T, const N: usize> IndexMut<RangeFull> for StaticVec<T, { N }> {
   ///Returns a mutable reference to a slice consisting of `0..self.length`
-  //elements of the StaticVec, using [as_mut_slice](crate::StaticVec::as_mut_slice) internally.
+  ///elements of the StaticVec, using [as_mut_slice](crate::StaticVec::as_mut_slice) internally.
   #[inline(always)]
   fn index_mut(&mut self, _index: RangeFull) -> &mut Self::Output {
     self.as_mut_slice()
@@ -237,7 +266,7 @@ impl<T, const N: usize> IndexMut<RangeFull> for StaticVec<T, { N }> {
 impl<T, const N: usize> Index<RangeInclusive<usize>> for StaticVec<T, { N }> {
   type Output = [T];
   ///Asserts that the lower bound of `index` is less than or equal to its upper bound,
-  //and that its upper bound is less than the current length of the StaticVec,
+  ///and that its upper bound is less than the current length of the StaticVec,
   ///and if so returns a constant reference to a slice of elements `index.start()..=index.end()`.
   #[allow(clippy::op_ref)]
   #[inline(always)]
@@ -249,7 +278,7 @@ impl<T, const N: usize> Index<RangeInclusive<usize>> for StaticVec<T, { N }> {
 
 impl<T, const N: usize> IndexMut<RangeInclusive<usize>> for StaticVec<T, { N }> {
   ///Asserts that the lower bound of `index` is less than or equal to its upper bound,
-  //and that its upper bound is less than the current length of the StaticVec,
+  ///and that its upper bound is less than the current length of the StaticVec,
   ///and if so returns a mutable reference to a slice of elements `index.start()..=index.end()`.
   #[allow(clippy::op_ref)]
   #[inline(always)]
@@ -327,49 +356,121 @@ impl_partial_ord_with_as_slice_against_slice!([T1], &mut StaticVec<T2, { N }>);
 impl_partial_ord_with_as_slice_against_slice!(&[T1], StaticVec<T2, { N }>);
 impl_partial_ord_with_as_slice_against_slice!(&mut [T1], StaticVec<T2, { N }>);
 
+/// Read from a [`StaticVec`]. This implementation reads from the `StaticVec`
+/// by copying bytes into the destination buffers, then shifting the remaining
+/// bytes over. This might be inefficient for your needs; consider using
+/// [`Cursor`] or [`[T] as Read`][slice-read] for more efficient
+/// ways to read out of a `StaticVec` without mutating it.
+///
+/// [`Cursor`]: https://doc.rust-lang.org/stable/std/io/struct.Cursor.html
+/// [slice-read]: https://doc.rust-lang.org/stable/std/primitive.slice.html#impl-Read]
 #[cfg(feature = "std")]
 impl<const N: usize> Read for StaticVec<u8, { N }> {
   #[inline(always)]
+  unsafe fn initializer(&self) -> io::Initializer {
+    io::Initializer::nop()
+  }
+
+  #[inline]
   fn read(&mut self, buf: &mut [u8]) -> io::Result<usize> {
     let read_length = self.length.min(buf.len());
+    // Safety:  read_length <= buf.length and self.length. Rust borrowing
+    // rules mean that buf is guaranteed not to overlap with self.
     unsafe {
-      self
-        .drain(0..read_length)
-        .as_ptr()
-        .copy_to_nonoverlapping(buf.as_mut_ptr(), read_length);
+      buf
+        .as_mut_ptr()
+        .copy_from_nonoverlapping(self.as_ptr(), read_length);
     }
+
+    if read_length < self.length {
+      // TODO: find out if the optimizer elides the bounds check here. It
+      // should be able to, since the only non-const value is read_length,
+      // which is known to be <= self.length
+      self.as_mut_slice().copy_within(read_length.., 0);
+    }
+    // Safety: 0 <= read_length <= self.length
+    self.length -= read_length;
     Ok(read_length)
   }
 
-  #[inline(always)]
+  #[inline]
+  fn read_to_end(&mut self, buf: &mut Vec<u8>) -> io::Result<usize> {
+    let read_length = self.length;
+    buf.extend_from_slice(self.as_slice());
+    self.length = 0;
+    Ok(read_length)
+  }
+
+  #[inline]
+  fn read_to_string(&mut self, buf: &mut String) -> io::Result<usize> {
+    let read_length = self.length;
+    match str::from_utf8(self.as_slice()) {
+      Err(err) => return Err(io::Error::new(io::ErrorKind::InvalidData, err)),
+      Ok(self_str) => buf.push_str(self_str),
+    };
+    self.length = 0;
+    Ok(read_length)
+  }
+
+  #[inline]
   fn read_exact(&mut self, buf: &mut [u8]) -> io::Result<()> {
     if buf.len() > self.length {
-      return Err(Error::new(
-        ErrorKind::UnexpectedEof,
+      Err(io::Error::new(
+        io::ErrorKind::UnexpectedEof,
         "Not enough data available to fill the provided buffer!",
-      ));
+      ))
+    } else {
+      // read is guaranteed to fully read into the buf in a single call
+      self.read(buf).and(Ok(()))
     }
-    //Our implementation of `read` always returns `Ok(read_length)`, so we can unwrap safely here.
-    self.read(buf).unwrap();
-    Ok(())
   }
 
   #[inline]
   fn read_vectored(&mut self, bufs: &mut [IoSliceMut]) -> io::Result<usize> {
-    if self.is_empty() {
-      return Ok(0);
-    }
-    let mut read_length = 0;
+    // Minimize copies: copy to each output buf in sequence, then shfit the
+    // internal data only once. This as opposed to calling `read` in a loop,
+    // which shifts the inner data each time.
+    let mut start_ptr = self.as_ptr();
+    let original_length = self.length;
+
+    // We update self.length inplace in the loop to track how many bytes
+    // have been written. This means that when we perform the shift at the
+    // end, self.length is already correct.
     for buf in bufs {
-      read_length += self.read(buf).unwrap();
+      if self.length == 0 {
+        break;
+      }
+
+      // The number of bytes we'll be reading out of self
+      let read_length = self.length.min(buf.len());
+
+      // Safety: start_ptr is known to point to the array in self, which
+      // is different than `buf`. read_length <= self.length.
+      unsafe {
+        buf
+          .as_mut_ptr()
+          .copy_from_nonoverlapping(start_ptr, read_length);
+        start_ptr = start_ptr.add(read_length);
+        self.length -= read_length;
+      }
     }
-    Ok(read_length)
+
+    let total_read = original_length - self.length;
+
+    if self.length > 0 {
+      // TODO: find out if the optimizer elides the bounds check here. It
+      // should be able to, since the only non-const value is total_read,
+      // which is known to be <= self.length
+      self.as_mut_slice().copy_within(total_read.., 0);
+    }
+
+    Ok(total_read)
   }
 }
 
 #[cfg(feature = "std")]
 impl<const N: usize> Write for StaticVec<u8, { N }> {
-  #[inline(always)]
+  #[inline]
   fn write(&mut self, buf: &[u8]) -> io::Result<usize> {
     let old_length = self.length;
     self.extend_from_slice(buf);
@@ -380,19 +481,22 @@ impl<const N: usize> Write for StaticVec<u8, { N }> {
   fn write_vectored(&mut self, bufs: &[IoSlice<'_>]) -> io::Result<usize> {
     let old_length = self.length;
     for buf in bufs {
+      if self.is_full() {
+        break;
+      }
       self.extend_from_slice(buf);
     }
     Ok(self.length - old_length)
   }
 
-  #[inline(always)]
+  #[inline]
   fn write_all(&mut self, buf: &[u8]) -> io::Result<()> {
-    //Our implementation of `write` always returns `Ok(self.length - old_length)`, so we can unwrap safely here.
-    if self.write(buf).unwrap() == buf.len() {
+    if buf.len() <= self.remaining_capacity() {
+      self.extend_from_slice(buf);
       Ok(())
     } else {
-      Err(Error::new(
-        ErrorKind::WriteZero,
+      Err(io::Error::new(
+        io::ErrorKind::WriteZero,
         "Insufficient remaining capacity!",
       ))
     }
