@@ -16,7 +16,7 @@ impl Drop for Struct {
 }
 
 #[cfg(feature = "std")]
-use std::io::{IoSlice, IoSliceMut, Read, Write};
+use std::io::{IoSlice, Write};
 
 #[test]
 fn as_mut_ptr() {
@@ -79,6 +79,24 @@ fn clone() {
   assert_eq!(v, vv);
 }
 
+#[test]
+fn clone_from_shorter() {
+  let src: StaticVec<u32, { 20 }> = (1..10).collect();
+  let mut dst: StaticVec<u32, { 20 }> = (0..15).collect();
+
+  dst.clone_from(&src);
+  assert_eq!(dst, src);
+}
+
+#[test]
+fn clone_from_longer() {
+  let src: StaticVec<u32, { 20 }> = (0..15).collect();
+  let mut dst: StaticVec<u32, { 20 }> = (1..10).collect();
+
+  dst.clone_from(&src);
+  assert_eq!(dst, src);
+}
+
 #[test]
 fn dedup() {
   let mut vec = staticvec![1, 2, 2, 3, 2];
@@ -183,6 +201,18 @@ fn from() {
   assert_eq!(StaticVec::<i32, 6>::from(v.as_slice()).len(), 0);
 }
 
+#[test]
+fn get_unchecked() {
+  let v = staticvec!["a", "b", "c"];
+  assert_eq!(unsafe { *v.get_unchecked(1) }, "b");
+}
+
+#[test]
+fn get_unchecked_mut() {
+  let mut v = staticvec!["a", "b", "c"];
+  assert_eq!(unsafe { *v.get_unchecked_mut(1) }, "b");
+}
+
 #[test]
 fn index() {
   let vec = staticvec![0, 1, 2, 3, 4];
@@ -356,49 +386,84 @@ fn push() {
 }
 
 #[cfg(feature = "std")]
-#[test]
-fn read() {
-  let mut ints = staticvec![1, 2, 3, 4, 6, 7, 8, 9, 10];
-  let mut buffer = [0, 0, 0, 0];
-  assert_eq!(ints.read(&mut buffer).unwrap(), 4);
-  assert_eq!(buffer, [1, 2, 3, 4]);
-  let mut buffer2 = [];
-  assert_eq!(ints.read(&mut buffer2).unwrap(), 0);
-  assert_eq!(buffer2, []);
-  let mut buffer3 = staticvec![0; 9];
-  assert_eq!(ints.read(buffer3.as_mut_slice()).unwrap(), 5);
-  assert_eq!(ints, []);
-  assert_eq!(ints.read(staticvec![].as_mut_slice()).unwrap(), 0);
-}
+mod read_tests {
+  use staticvec::*;
+  use std::io::{self, Read};
+  // We provide custom implementations of most `Read` methods; test those
+  // impls
+  #[test]
+  fn read() {
+    let mut ints = staticvec![1, 2, 3, 4, 6, 7, 8, 9, 10];
+    let mut buffer = [0, 0, 0, 0];
+    assert_eq!(ints.read(&mut buffer).unwrap(), 4);
+    assert_eq!(buffer, [1, 2, 3, 4]);
+    let mut buffer2 = [];
+    assert_eq!(ints.read(&mut buffer2).unwrap(), 0);
+    assert_eq!(buffer2, []);
+    let mut buffer3 = staticvec![0; 9];
+    assert_eq!(ints.read(buffer3.as_mut_slice()).unwrap(), 5);
+    assert_eq!(ints, []);
+    assert_eq!(ints.read(staticvec![].as_mut_slice()).unwrap(), 0);
+  }
 
-#[cfg(feature = "std")]
-#[test]
-fn read_exact() {
-  let mut ints = staticvec![1, 2, 3, 4, 6, 7, 8, 9, 10];
-  let mut buffer = [0, 0, 0, 0];
-  ints.read_exact(&mut buffer).unwrap();
-  assert_eq!(buffer, [1, 2, 3, 4]);
-  let mut buffer2 = [0, 0, 0, 0, 0, 0, 0, 0];
-  assert!(ints.read_exact(&mut buffer2).is_err());
-}
+  #[test]
+  fn read_to_end() {
+    let mut ints = staticvec![1, 2, 3, 4, 5, 6, 7];
+    let mut buffer = vec![2, 3];
+    assert_eq!(ints.read_to_end(&mut buffer).unwrap(), 7);
+    assert_eq!(ints, &[]);
+    assert_eq!(buffer, &[2, 3, 1, 2, 3, 4, 5, 6, 7]);
+  }
 
-#[cfg(feature = "std")]
-#[test]
-fn read_vectored() {
-  let mut ints = staticvec![1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12];
-  let mut buf1 = [0; 4];
-  let mut buf2 = [0; 4];
-  let mut buf3 = [0; 4];
-  let bufs = &mut [
-    IoSliceMut::new(&mut buf1),
-    IoSliceMut::new(&mut buf2),
-    IoSliceMut::new(&mut buf3),
-  ];
-  assert_eq!(ints.read_vectored(bufs).unwrap(), 12);
-  assert_eq!(
-    "[[1, 2, 3, 4], [5, 6, 7, 8], [9, 10, 11, 12]]",
-    format!("{:?}", bufs)
-  );
+  #[test]
+  fn read_to_string() {
+    // Hello world in ascii
+    let mut input = StaticVec::<u8, 30>::new_from_slice(b"World!");
+    let mut dest = String::from("Hello, ");
+    assert_eq!(input.read_to_string(&mut dest).unwrap(), 6);
+    assert_eq!(dest, "Hello, World!");
+    assert_eq!(input, &[]);
+  }
+
+  #[test]
+  fn read_to_string_failure() {
+    // Invalid UTF-8 bytes
+    let mut input = staticvec![0b1101_1010, 0b1100_0000];
+    let mut dest = String::new();
+    let err = input.read_to_string(&mut dest).unwrap_err();
+    assert_eq!(err.kind(), io::ErrorKind::InvalidData);
+  }
+
+  #[test]
+  fn read_exact() {
+    let mut ints = staticvec![1, 2, 3, 4, 6, 7, 8, 9, 10];
+    let mut buffer = [0, 0, 0, 0];
+    ints.read_exact(&mut buffer).unwrap();
+    assert_eq!(buffer, [1, 2, 3, 4]);
+    assert_eq!(ints, &[6, 7, 8, 9, 10]);
+
+    let mut buffer2 = [0, 0, 0, 0, 0, 0, 0, 0];
+    let err = ints.read_exact(&mut buffer2).unwrap_err();
+    assert_eq!(err.kind(), io::ErrorKind::UnexpectedEof);
+  }
+
+  #[test]
+  fn read_vectored() {
+    let mut ints = staticvec![1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12];
+    let mut buf1 = [0; 4];
+    let mut buf2 = [0; 4];
+    let mut buf3 = [0; 4];
+    let bufs = &mut [
+      io::IoSliceMut::new(&mut buf1),
+      io::IoSliceMut::new(&mut buf2),
+      io::IoSliceMut::new(&mut buf3),
+    ];
+    assert_eq!(ints.read_vectored(bufs).unwrap(), 12);
+    assert_eq!(
+      "[[1, 2, 3, 4], [5, 6, 7, 8], [9, 10, 11, 12]]",
+      format!("{:?}", bufs)
+    );
+  }
 }
 
 #[test]
@@ -431,13 +496,6 @@ fn retain() {
   assert_eq!(vec, [2, 3, 5]);
 }
 
-#[test]
-fn reverse() {
-  let mut v = staticvec![1, 2, 3];
-  v.reverse();
-  assert!(v == [3, 2, 1]);
-}
-
 #[test]
 fn reversed() {
   let v = staticvec![1, 2, 3].reversed();
@@ -452,14 +510,6 @@ fn set_len() {
   assert_eq!(v.len(), 0);
 }
 
-#[cfg(feature = "std")]
-#[test]
-fn sort() {
-  let mut v = staticvec![-5, 4, 1, -3, 2];
-  v.sort();
-  assert!(v == [-5, -3, 1, 2, 4]);
-}
-
 #[cfg(feature = "std")]
 #[test]
 fn sorted() {
@@ -467,13 +517,6 @@ fn sorted() {
   assert!(v == [-5, -3, 1, 2, 4]);
 }
 
-#[test]
-fn sort_unstable() {
-  let mut v = staticvec![-5, 4, 1, -3, 2];
-  v.sort_unstable();
-  assert!(v == [-5, -3, 1, 2, 4]);
-}
-
 #[test]
 fn sorted_unstable() {
   let v = staticvec![-5, 4, 1, -3, 2].sorted_unstable();
